==> oceanbase-client-2.4.0-sources.jar: com/oceanbase/jdbc/internal/com/read/resultset/SensitiveCursorResultSet.java <==
/**
 *  OceanBase Client for Java
 *
 *  Copyright (c) 2012-2014 Monty Program Ab.
 *  Copyright (c) 2015-2020 MariaDB Corporation Ab.
 *  Copyright (c) 2021 OceanBase.
 *
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 2.1 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to Monty Program Ab info@montyprogram.com.
 *
 *  This particular MariaDB Client for Java file is work
 *  derived from a Drizzle-JDBC. Drizzle-JDBC file which is covered by subject to
 *  the following copyright and notice provisions:
 *
 *  Copyright (c) 2009-2011, Marcus Eriksson
 *
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *  Redistributions of source code must retain the above copyright notice, this list
 *  of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice, this
 *  list of conditions and the following disclaimer in the documentation and/or
 *  other materials provided with the distribution.
 *
 *  Neither the name of the driver nor the names of its contributors may not be
 *  used to endorse or promote products derived from this software without specific
 *  prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS  AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *  OF SUCH DAMAGE.
 */
package com.oceanbase.jdbc.internal.com.read.resultset;

import java.io.IOException;
import java.sql.SQLException;

import com.oceanbase.jdbc.internal.com.read.dao.Results;
import com.oceanbase.jdbc.internal.protocol.Protocol;

public class SensitiveCursorResultSet extends CursorResultSet {

    private int lastRefetchBeginIndex;
    private int lastRefetchEndIndex;

    public SensitiveCursorResultSet(ColumnDefinition[] columnsInformation, Results results,
                                    Protocol protocol, boolean callableResult,
                                    boolean eofDeprecated, boolean isPsOutParamter)
                                                                                   throws IOException,
                                                                                   SQLException {
        super(columnsInformation, results, protocol, callableResult, eofDeprecated, isPsOutParamter);

        if (this.protocol.supportStmtPrepareExecute()) {
            lastRefetchBeginIndex = 0;
            lastRefetchEndIndex = fetchSize - 1;
        } else {
            lastRefetchBeginIndex = -1;
            lastRefetchEndIndex = -1;
        }
    }

    @Override
    public boolean next() throws SQLException {
        if (super.next()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean previous() throws SQLException {
        if (super.previous()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean first() throws SQLException {
        if (super.first()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean last() throws SQLException {
        if (super.last()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean absolute(int row) throws SQLException {
        if (super.absolute(row)) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean relative(int row) throws SQLException {
        if (super.relative(row)) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    private boolean needRefetch() throws SQLException {
        if (rowPointer < lastRefetchBeginIndex && rowPointer < lastRefetchEndIndex
            || rowPointer > lastRefetchEndIndex && rowPointer > lastRefetchBeginIndex) {
            refreshRow();
            return true;
        } else {
            return false;
        }
    }

    /**
     * SQLException –
     *  if a database access error occurs;
     *  this method is called on a closed result set;
     *  the result set type is TYPE_FORWARD_ONLY, or (TYPE_SCROLL_INSENSITIVE and CONCUR_READ_ONLY)
     *  or if this method is called when the cursor is on the insert row
     *
     * support FETCH_FORWARD direction for Oracle mode now
     */
    @Override
    public void refreshRow() throws SQLException {
        if (!checkRefreshable()) {
            return;
        }

        int refetchSize = super.refreshRowInternalOracle(fetchSize);
        lastRefetchBeginIndex = rowPointer;
        lastRefetchEndIndex = rowPointer + refetchSize - 1;
    }

}

==> oceanbase-client-2.4.9-sources.jar: com/oceanbase/jdbc/internal/com/read/resultset/SensitiveCursorResultSet.java <==
/**
 *  OceanBase Client for Java
 *
 *  Copyright (c) 2012-2014 Monty Program Ab.
 *  Copyright (c) 2015-2020 MariaDB Corporation Ab.
 *  Copyright (c) 2021 OceanBase.
 *
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 2.1 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to Monty Program Ab info@montyprogram.com.
 *
 *  This particular MariaDB Client for Java file is work
 *  derived from a Drizzle-JDBC. Drizzle-JDBC file which is covered by subject to
 *  the following copyright and notice provisions:
 *
 *  Copyright (c) 2009-2011, Marcus Eriksson
 *
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *  Redistributions of source code must retain the above copyright notice, this list
 *  of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice, this
 *  list of conditions and the following disclaimer in the documentation and/or
 *  other materials provided with the distribution.
 *
 *  Neither the name of the driver nor the names of its contributors may not be
 *  used to endorse or promote products derived from this software without specific
 *  prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS  AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *  OF SUCH DAMAGE.
 */
package com.oceanbase.jdbc.internal.com.read.resultset;

import java.io.IOException;
import java.sql.SQLException;

import com.oceanbase.jdbc.internal.com.read.dao.Results;
import com.oceanbase.jdbc.internal.protocol.Protocol;

public class SensitiveCursorResultSet extends CursorResultSet {

    private int lastRefetchBeginIndex;
    private int lastRefetchEndIndex;

    public SensitiveCursorResultSet(ColumnDefinition[] columnsInformation, Results results,
                                    Protocol protocol, boolean callableResult,
                                    boolean eofDeprecated, boolean isPsOutParamter)
                                                                                   throws IOException,
                                                                                   SQLException {
        super(columnsInformation, results, protocol, callableResult, eofDeprecated, isPsOutParamter);

        if (this.protocol.supportStmtPrepareExecute()) {
            lastRefetchBeginIndex = 0;
            lastRefetchEndIndex = fetchSize - 1;
        } else {
            lastRefetchBeginIndex = -1;
            lastRefetchEndIndex = -1;
        }
    }

    @Override
    public boolean next() throws SQLException {
        if (super.next()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean previous() throws SQLException {
        if (super.previous()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean first() throws SQLException {
        if (super.first()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean last() throws SQLException {
        if (super.last()) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean absolute(int row) throws SQLException {
        if (super.absolute(row)) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean relative(int row) throws SQLException {
        if (super.relative(row)) {
            needRefetch();
            return true;
        } else {
            return false;
        }
    }

    private boolean needRefetch() throws SQLException {
        if (rowPointer < lastRefetchBeginIndex && rowPointer < lastRefetchEndIndex
            || rowPointer > lastRefetchEndIndex && rowPointer > lastRefetchBeginIndex) {
            refreshRow();
            return true;
        } else {
            return false;
        }
    }

    /**
     * SQLException –
     *  if a database access error occurs;
     *  this method is called on a closed result set;
     *  the result set type is TYPE_FORWARD_ONLY, or (TYPE_SCROLL_INSENSITIVE and CONCUR_READ_ONLY)
     *  or if this method is called when the cursor is on the insert row
     *
     * support FETCH_FORWARD direction for Oracle mode now
     */
    @Override
    public void refreshRow() throws SQLException {
        if (!checkRefreshable()) {
            return;
        }

        int refetchSize = refreshRowInternalOracle(fetchSize);
        lastRefetchBeginIndex = rowPointer;
        lastRefetchEndIndex = rowPointer + refetchSize - 1;
    }

}
